#!/usr/bin/python
# encoding: utf-8
#
# Copyright (c) 2020 Dean Jackson <deanishe@deanishe.net>
# MIT Licence. See http://opensource.org/licenses/MIT
#
# Created on 2020-06-13
#

"""Show random images."""

from __future__ import print_function, absolute_import

import argparse
import json
import os
import random
import re
import subprocess
import sys
import time

# Seed random number generator
random.seed()

# Application to open images in
APP_NAME = os.getenv('APP_NAME') or 'Preview'
# Directory to load images from
IMAGE_DIR = os.path.expanduser(os.getenv('IMAGE_DIR') or '~/Pictures')
# Whether or not to play sounds
PLAY_SOUND = os.getenv('PLAY_SOUND', '').lower() in ('true', 'yes', '1')
# Extensions of image files
IMAGE_EXTS = ('.png', '.jpg', '.jpeg', '.tiff', '.tif')


ICON_WARNING = ('/System/Library/CoreServices/CoreTypes.bundle/'
                'Contents/Resources/AlertCautionIcon.icns')


AS_PREVIEW_FULLSCREEN = """
on openFullscreen(_path)
    tell application "Preview"
        activate
        open (_path as POSIX file)
        -- wait for up to 5 seconds for Preview to open the image
        repeat 50 times
            set _winpath to (path of document of window 1) as text
            -- log "_path=" & _path & ", _winpath=" & _winpath
            if _winpath = _path then
                tell application "System Events" to tell process "Preview"
                    set value of attribute "AXFullScreen" of window 1 to true
                end tell
                return
            end if
            delay 0.1
        end repeat
        display dialog "Preview timed out" with icon stop ¬
            buttons {"OK"} default button 1 ¬
            giving up after 3
    end tell
end openFullscreen

on run (argv)
    openFullscreen(first item of argv)
end run
"""

AS_SOUND = """
tell application id "com.runningwithcrayons.Alfred"
    run trigger "sound" in workflow "net.deanishe.alfred.demo.random-images"
end tell
"""


def log(s, *args, **kwargs):
    """Log to STDERR."""
    if args:
        s = s % args
    elif kwargs:
        s = s % kwargs
    print(s, file=sys.stderr)


def is_image(path):
    """Return ``True`` if path points to an image file."""
    if os.path.isdir(path):
        return False
    _, ext = os.path.splitext(path)
    return ext.lower() in IMAGE_EXTS


def get_images(dirpath):
    """Load all image files from a directory."""
    images = [os.path.join(dirpath, name) for name in os.listdir(dirpath)]
    return [p for p in images if is_image(p)]


def parse_args():
    """Parse user input."""
    p = argparse.ArgumentParser()
    p.add_argument('--interval', type=int, help='delay between images')
    p.add_argument('--count', type=int, help='number of images to show')
    p.add_argument('query', nargs='?')
    return p.parse_args()


def parse_input(s):
    """Parse user input."""
    m = re.match(r'(\d+)\s+(\d+)', s.strip())
    if not m:
        raise ValueError

    return int(m.group(1)), int(m.group(2))


def warn(title, subtitle=None):
    """Show user a warning."""
    item = {
        'title': title,
        'subtitle': subtitle,
        'valid': False,
        'icon': {'path': ICON_WARNING},
    }
    print_items(item)


def print_items(*items):
    """Send JSON feedback to Alfred."""
    json.dump({'items': items}, sys.stdout)


def open_image(path):
    """Open image in application.

    If application is Preview, image is opened in fullscreen mode.

    """
    log('opening "%s" in "%s" ...', path, APP_NAME)
    if APP_NAME == 'Preview':
        cmd = ['/usr/bin/osascript', '-e', AS_PREVIEW_FULLSCREEN]
    else:
        cmd = ['/usr/bin/open', '-a', APP_NAME]

    subprocess.check_call(cmd + [path])


def play_sound():
    """Call External Trigger to play sound."""
    subprocess.check_call(['/usr/bin/osascript', '-e', AS_SOUND])


def main():
    """Run script."""
    args = parse_args()
    # ------------------------------------------------------------------
    # Script Filter
    # ------------------------------------------------------------------
    if args.query:
        log('query=%r', args.query)
        try:
            t, n = parse_input(args.query)
        except ValueError:
            warn('Invalid Input', u'Enter two numbers, e.g. “30 5”')
            return

        item = {
            'title': 'Show %d image(s) for %d second(s) each' % (n, t),
            'subtitle': u'Press ↩ to start',
            'arg': 'blah',
            'valid': True,
            'variables': {
                'interval': str(t),
                'count': str(n),
            }
        }
        print_items(item)
        return

    # ------------------------------------------------------------------
    # Open images
    # ------------------------------------------------------------------
    all_images = get_images(IMAGE_DIR)
    images = random.sample(all_images, args.count)
    log('loaded %d random image(s) from %s', len(images), IMAGE_DIR)

    for i, path in enumerate(images):
        open_image(path)

        # no point waiting and playing sounds for final image
        if i == len(images) - 1:
            break

        t = args.interval * 0.5
        time.sleep(t)
        if PLAY_SOUND:
            play_sound()
        time.sleep(t)
        if PLAY_SOUND:
            play_sound()


if __name__ == '__main__':
    sys.exit(main())
